#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <string.h>
#include <sys/utsname.h>
#include <sys/socket.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>

#define DEBUG_THEME

/* default positions for items */
#define CLOCK_LEFT 100
#define CLOCK_TOP 100

#define LOGO_LEFT 50
#define LOGO_TOP 50

#define INPUT_LEFT 50
#define INPUT_TOP 90
#define INPUT_BOUNDINGBOX 1

/* structure used for determining the actual widget sizes
   depending on gtkrc theme settings */
typedef struct {
    gint w;
    gint h;
} widgetsize;

/* structure used to carry actual on screen position values
   for widgets determined fro a procentual theme positioning
   value */
typedef struct {
    gint x;
    gint y;
} widgetposition;

static gboolean delete_event( GtkWidget *widget,
                              GdkEvent  *event,
                              gpointer   data )
{
    g_print ("delete event occurred\n");
    return TRUE;
}

static void destroy( GtkWidget *widget,
                     gpointer   data )
{
    gtk_main_quit ();
}

/* prints out the used size and position values for
   the theme item defined by name */
static void dbg_size_and_position(gchar *name,
		gint width,
		gint height,
		gint x,
		gint y)
{
	printf("%s Width: %u\n",name , width);
	printf("%s Height: %u\n",name , height);
	printf("%s X position: %u\n",name , x);
	printf("%s Y position: %u\n",name , y);
}

/* draws gtk widgets into a helper window to determine the
   actual on screen size of the widget taking gtkrc themeing
   into account, then reparents the widget into *taget,
   destroys the helper window and returns a widgetsize enum */
static widgetsize get_size_and_reparent( GtkWidget *widget,
		GtkWidget *target )
{
    GtkWidget *helperwin;
    widgetsize ret;

    helperwin = gtk_window_new(GTK_WINDOW_POPUP);
    gtk_window_set_keep_below(GTK_WINDOW(helperwin), TRUE);

    gtk_container_add(GTK_CONTAINER(helperwin), widget);

    gtk_widget_show_all(helperwin);

    ret.w = widget->allocation.width;
    ret.h = widget->allocation.height;

    gtk_widget_reparent(widget, target);
    gtk_widget_destroy(helperwin);

    return ret;
}

/* function to get the actual item position from a procentual
   value (xpos, ypos) depending on the actual screen size
   returning a widgetposition enum */
static widgetposition get_position_for_screensize(gint screenwidth,
		gint screenheight,
		gint xpos,
		gint ypos )
{
    widgetposition ret;

    ret.x = (screenwidth * xpos) / 100;
    ret.y = (screenheight * ypos) / 100;

    return ret;
}

static gboolean update_time(GtkWidget * label)
{
    gchar *timestring = 0;
    time_t timet;
    struct tm *timePtr;

    timet = time(NULL);
    timePtr = localtime(&timet);

    // use 12 hour clock format if LDM_12HOURCLOCK is set to true
/*    if (ldm_getenv_bool("LDM_12HOURCLOCK")) {
        timestring = g_strdup_printf("%.2d:%.2d ",
                                   (timePtr->tm_hour % 12) ? (timePtr->tm_hour % 12) : 12,
                                   timePtr->tm_min);
    }
    else {*/
        timestring = g_strdup_printf("%.2d:%.2d ",
                                   timePtr->tm_hour, timePtr->tm_min);
/*    }*/

    gtk_label_set_markup((GtkLabel *) label, timestring);

    g_free(timestring);

    return TRUE;
}

/* create user/pass input window */
static void inputbox( GtkWidget *widget,
		     gint x,
		     gint y )
{
    GtkWidget *UserPrompt, *entry, *vbox, *image;
    GdkPixbuf *bbox;
    gint draw_x, draw_y, padding, bw, bh, vx,vy;
    gchar *pic;
    widgetsize size;
    widgetposition pos;

    padding = 50;

    entry = gtk_entry_new();
    UserPrompt = gtk_label_new("");
    vbox = gtk_vbox_new(FALSE, 0);

    gtk_label_set_markup(GTK_LABEL(UserPrompt), "<span weight=\"bold\" fgcolor=\"#000000\">Username:</span>");
    gtk_misc_set_alignment((GtkMisc *) UserPrompt, 0, 0);

    gtk_entry_set_has_frame (GTK_ENTRY (entry), FALSE);

    gtk_box_pack_start(GTK_BOX(vbox), UserPrompt, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 5);

    pos = get_position_for_screensize(x, y, INPUT_LEFT, INPUT_TOP);
    size = get_size_and_reparent(vbox, widget);

    draw_x = pos.x + padding - size.w ;
    if (draw_x > x || draw_x < 0)
    {
	    draw_x = 0;
    }

    draw_y = pos.y + padding - size.h;

    if (draw_y > x || draw_y < 0)
    {
	    draw_y = 0;
    }

#ifdef DEBUG_THEME
    dbg_size_and_position("Input", size.w, size.h, draw_x, draw_y);
#endif

    if (INPUT_BOUNDINGBOX)
    {
	bw = size.w + padding;
	bh = size.h + padding;
	pic = "./boundingbox.png";
	bbox = gdk_pixbuf_new_from_file_at_scale(pic, bw, bh, FALSE, NULL);
	image = gtk_image_new_from_pixbuf(bbox);

	gtk_fixed_put (GTK_FIXED(widget), image, draw_x, draw_y);
    }

    vx = draw_x + (padding / 2);
    vy = draw_y + (padding / 2);

    gtk_fixed_move (GTK_FIXED(widget), vbox, vx, vy);
}

/* create a logo */
static void logo( GtkWidget *widget,
		     gint x,
		     gint y )
{
    GtkWidget *logo;
    GdkPixbuf *pix;
    gint lw, lh, draw_x, draw_y;
    widgetposition pos;

    logo = gtk_image_new_from_file("./logo.png");
    pix = gtk_image_get_pixbuf((GtkImage *) logo);

    lw = gdk_pixbuf_get_width(pix);
    lh = gdk_pixbuf_get_height(pix);

    pos = get_position_for_screensize(x, y, LOGO_LEFT, LOGO_TOP);

    draw_x = pos.x - (lw / 2);

    if (draw_x > x || draw_x < 0)
    {
	    draw_x = 0;
    }

    draw_y = pos.y - (lh / 2);

    if (draw_y > x || draw_y < 0)
    {
	    draw_y = 0;
    }

#ifdef DEBUG_THEME
    dbg_size_and_position("Logo", lw, lh, draw_x, draw_y);
#endif

    gtk_fixed_put (GTK_FIXED(widget), logo, draw_x, draw_y);
}

/* create a digital clock */
static void clockapp( GtkWidget *widget,
		     gint x,
		     gint y )
{
    GtkWidget *timelabel, *timebox;
    widgetsize size;
    widgetposition pos;
    gint draw_x, draw_y;

    timelabel = gtk_label_new("");
    timebox = gtk_vbox_new(FALSE, 0);

    gtk_box_pack_start(GTK_BOX(timebox), timelabel, TRUE, TRUE, 0);

    update_time(timelabel);

    size = get_size_and_reparent(timebox, widget);
    pos = get_position_for_screensize(x, y, CLOCK_LEFT, CLOCK_TOP);

    draw_x = pos.x - size.w;
    draw_y = pos.y - size.h;

#ifdef DEBUG_THEME
    dbg_size_and_position("Clock", size.w, size.h, draw_x, draw_y);
#endif

    gtk_fixed_move (GTK_FIXED(widget), timebox, draw_x, draw_y);

    g_timeout_add(30000, (GSourceFunc) update_time, timelabel);
}

static void optionmenu( GtkWidget *widget,
		     gint x,
		     gint y )
{

}

static void hostname( GtkWidget *widget,
		     gint x,
		     gint y )
{

}

static void bottombar( GtkWidget *widget,
		     gint x,
		     gint y )
{

}

static void statusmessages( GtkWidget *widget,
		     gint x,
		     gint y )
{

}

static void timeoutbox( GtkWidget *widget,
		     gint x,
		     gint y )
{

}

static void guestbutton( GtkWidget *widget,
		     gint x,
		     gint y )
{

}

int main( int   argc,
	  char *argv[] )
{
    GtkWidget *window, *layout;
    GdkWindow *root;
    GdkPixmap *pic;
    GdkBitmap *mask;
    GdkPixbuf *rawpic;
    gint width, height, w, h, pw, ph;

    gtk_init (&argc, &argv);

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    layout = gtk_fixed_new ();

    root = gdk_get_default_root_window();
    gdk_drawable_get_size(root, &width, &height);

    w = width;
    h = height;
    pw = width;
    ph = height;

    rawpic = gdk_pixbuf_new_from_file_at_scale("./bg.png", pw, ph, FALSE, NULL);
    gdk_pixbuf_render_pixmap_and_mask(rawpic, &pic, &mask, 0);
    gdk_pixbuf_unref(rawpic);

    gtk_widget_set_app_paintable(window, TRUE);
    gtk_widget_set_size_request(window, w, h);
    gtk_widget_realize(window);
    gdk_window_set_back_pixmap((GdkWindow *) window->window, pic, 0);
    gtk_window_set_decorated((GtkWindow *) window, FALSE);

    g_signal_connect (G_OBJECT (window), "delete_event",
		      G_CALLBACK (delete_event), NULL);
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (destroy), NULL);

    /* TODO add switches to turn on/off elements */
    logo(layout, w , h);
    inputbox(layout, w , h);
    clockapp(layout, w, h);

    gtk_container_add(GTK_CONTAINER(window), layout);
    gtk_widget_show_all (window);

    gtk_main ();
    return 0;
}
